@@ -1,14 +1,13 @@
 from __future__ import annotations
 
 import asyncio
-import copy
 import logging
 import marshal
 import os
 import sys
 from enum import Enum, unique
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any, Dict, List, Optional
 from xmlrpc.client import boolean
 
 import jsonschema
@@ -16,7 +15,7 @@ import jsonschema
 from datayoga_core import blocks, utils
 from datayoga_core.block import Block
 from datayoga_core.context import Context
-from datayoga_core.result import Result, Status
+from datayoga_core.result import JobResult, Result, Status
 from datayoga_core.step import Step
 
 logger = logging.getLogger("dy")
@@ -26,7 +25,6 @@ class ErrorHandling(str, Enum):
     ABORT = "abort"
     IGNORE = "ignore"
 
-
 class Job():
     """
     Job
@@ -71,9 +69,7 @@ class Job():
         self.root.add_done_callback(self.handle_results)
         self.initialized = True
 
-    def transform(self, data: List[Dict[str, Any]],
-                  deepcopy: boolean = True) -> Tuple[List[Dict[str, Any]],
-                                                     List[Result]]:
+    def transform(self, data: List[Dict[str, Any]], deepcopy: boolean = True) -> JobResult:
         """
         Transforms data
 
@@ -82,7 +78,7 @@ class Job():
             deepcopy: if True, performs a deepcopy before modifying records. otherwise, modifies in place. can affect performance.
 
         Returns:
-            Tuple[List[Dict[str, Any]], List[Result]]: Transformed data and results
+            JobResult: Job result
         """
         if not self.initialized:
             logger.debug("job has not been initialized yet, initializing...")
@@ -90,21 +86,32 @@ class Job():
         # use marshal. faster than deepcopy
         transformed_data = marshal.loads(marshal.dumps(data)) if deepcopy else data
 
-        results = []
-        for step in self.steps:
-            try:
-                transformed_data, results = asyncio.run(step.block.run(transformed_data))
-            except ConnectionError as e:
-                # connection errors are thrown back to the caller to handle
-                raise e
-            except Exception as e:
-                # other exceptions are rejected
-                logger.error(f"Error while transforming data: {e}")
-                utils.reject_records(data, f"{e}")
-                transformed_data, results = utils.produce_data_and_results(data)
-                break
+        result = JobResult()
+        # create an event loop for the duration of the transformation
+        loop = asyncio.new_event_loop()
 
-        return transformed_data, results
+        try:
+            for step in self.steps:
+                try:
+                    processed, filtered, rejected = loop.run_until_complete(step.block.run(transformed_data))
+                    result.filtered.extend(filtered)
+                    result.rejected.extend(rejected)
+                    transformed_data = [result.payload for result in processed]
+                except ConnectionError as e:
+                    # connection errors are thrown back to the caller to handle
+                    raise e
+                except Exception as e:
+                    # other exceptions are rejected
+                    logger.error(f"Error while transforming data: {e}")
+                    result.rejected.extend(
+                        [Result(Status.REJECTED, payload=row, message=f"{e}") for row in transformed_data])
+                    return result
+
+            result.processed.extend([Result(Status.SUCCESS, payload=row) for row in transformed_data])
+        finally:
+            # close the event loop
+            loop.close()
+        return result
 
     async def run(self):
         for record in self.input.produce():
